"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dom_1 = require("../dom");
const rule_1 = require("../rule");
const selector = ["h1", "h2", "h3", "h4", "h5", "h6"].join(",");
var TextClassification;
(function (TextClassification) {
    TextClassification[TextClassification["EMPTY_TEXT"] = 0] = "EMPTY_TEXT";
    TextClassification[TextClassification["DYNAMIC_TEXT"] = 1] = "DYNAMIC_TEXT";
    TextClassification[TextClassification["STATIC_TEXT"] = 2] = "STATIC_TEXT";
})(TextClassification || (TextClassification = {}));
class EmptyHeading extends rule_1.Rule {
    documentation() {
        return {
            description: `Assistive technology such as screen readers require textual content in headings. Whitespace only is considered empty.`,
            url: rule_1.ruleDocumentationUrl(__filename),
        };
    }
    setup() {
        this.on("dom:ready", ({ document }) => {
            const headings = document.querySelectorAll(selector);
            for (const heading of headings) {
                const text = this.findTextNodes(heading);
                switch (this.classifyText(text)) {
                    case TextClassification.DYNAMIC_TEXT:
                    case TextClassification.STATIC_TEXT:
                        break;
                    case TextClassification.EMPTY_TEXT:
                        this.report(heading, `<${heading.tagName}> cannot be empty, must have text content`);
                        break;
                }
            }
        });
    }
    findTextNodes(node) {
        let text = [];
        for (const child of node.childNodes) {
            switch (child.nodeType) {
                case dom_1.NodeType.TEXT_NODE:
                    text.push(child);
                    break;
                case dom_1.NodeType.ELEMENT_NODE:
                    text = text.concat(this.findTextNodes(child));
                    break;
                default:
                    break;
            }
        }
        return text;
    }
    classifyText(text) {
        for (const node of text) {
            if (node.isDynamic) {
                return TextClassification.DYNAMIC_TEXT;
            }
            if (node.textContent.match(/\S/)) {
                return TextClassification.STATIC_TEXT;
            }
        }
        return TextClassification.EMPTY_TEXT;
    }
}
module.exports = EmptyHeading;
